#include <windows.h>
#include <stdio.h>
#include "Configuration.hpp"
#include "Keys.hpp"
#include "Audio.hpp"


// driver interfaces: do not expose!

static bool  (__stdcall * SA_InitDriver)(HWND window);
static void  (__stdcall * SA_CloseDriver)();

static SA_SOUND (__stdcall * SA_LoadSound)(const char* filename);
static void     (__stdcall * SA_DestroySound)(SA_SOUND s);
static bool     (__stdcall * SA_PlaySound)(SA_SOUND s, bool repeat);
static void     (__stdcall * SA_StopSound)(SA_SOUND s);
static void     (__stdcall * SA_SetVolume)(SA_SOUND s, int volume);
static int      (__stdcall * SA_GetVolume)(SA_SOUND s);
static int      (__stdcall * SA_GetLength)(SA_SOUND sound);
static void     (__stdcall * SA_SetPosition)(SA_SOUND sound, int position);
static int      (__stdcall * SA_GetPosition)(SA_SOUND sound);
static int      (__stdcall * SA_IsPlaying)(SA_SOUND sound);


static void InitDriver();
static void CloseDriver();


static HINSTANCE AudioDriver;
static HWND EditorWindow;

static int s_RefCount;


////////////////////////////////////////////////////////////////////////////////

template<typename T>
static void assign(T& dest, void* src)
{
  dest = reinterpret_cast<T&>(src);
}

bool InitializeAudio(HWND window, const char* driver)
{
  EditorWindow = window;

  char audiodriver[MAX_PATH];
  sprintf(audiodriver, "system/audio/%s", driver);

  // Loads driver
  AudioDriver = LoadLibrary(audiodriver);
  if (AudioDriver == NULL)
    return false;  // silently fail

  // Gets addresses of all of the audio functions

#define get(x) {                              \
  assign(x, GetProcAddress(AudioDriver, #x)); \
  if (!x) {                                   \
    FreeLibrary(AudioDriver);                 \
    AudioDriver = NULL;                       \
    return true;  /* silently fail */         \
  }                                           \
}

  get(SA_InitDriver)
  get(SA_CloseDriver)

  get(SA_LoadSound)
  get(SA_DestroySound)
  get(SA_PlaySound)
  get(SA_StopSound)
  get(SA_SetVolume)
  get(SA_GetVolume)
  get(SA_GetLength)
  get(SA_SetPosition)
  get(SA_GetPosition)
  get(SA_IsPlaying)

#undef get

  s_RefCount = 0;

  return true;
}

////////////////////////////////////////////////////////////////////////////////

bool CloseAudio()
{
  CloseDriver();
  FreeLibrary(AudioDriver);
  AudioDriver = NULL;
  return true;
}

////////////////////////////////////////////////////////////////////////////////

void InitDriver()
{
  if (s_RefCount++ == 0) {
    if (!SA_InitDriver(EditorWindow)) {
      s_RefCount--;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////

void CloseDriver()
{
  if (--s_RefCount == 0) {
    SA_CloseDriver();
  }
}

////////////////////////////////////////////////////////////////////////////////

CSound::CSound()
: m_Sound(NULL)
{
  InitDriver();
}

////////////////////////////////////////////////////////////////////////////////

CSound::~CSound()
{
  if (m_Sound) {
    SA_DestroySound(m_Sound);
  }
  CloseDriver();
}

////////////////////////////////////////////////////////////////////////////////

bool
CSound::Load(const char* filename)
{
  if (m_Sound) {
    SA_DestroySound(m_Sound);
  }

  m_Sound = SA_LoadSound(filename);
  return (m_Sound != NULL);
}

////////////////////////////////////////////////////////////////////////////////

bool
CSound::Play()
{
  SA_PlaySound(m_Sound, false);
  return true;
}

////////////////////////////////////////////////////////////////////////////////

bool
CSound::Stop()
{
  SA_StopSound(m_Sound);
  return true;
}

////////////////////////////////////////////////////////////////////////////////

int
CSound::GetVolume()
{
  return SA_GetVolume(m_Sound);
}

////////////////////////////////////////////////////////////////////////////////

void
CSound::SetVolume(int Volume)
{
  SA_SetVolume(m_Sound, Volume);
}

////////////////////////////////////////////////////////////////////////////////

int 
CSound::GetSongLength()
{
  return SA_GetLength(m_Sound);
}

////////////////////////////////////////////////////////////////////////////////

int 
CSound::GetSongPos()
{
  return SA_GetPosition(m_Sound);
}

////////////////////////////////////////////////////////////////////////////////

void 
CSound::SetSongPos(int Pos)
{
  SA_SetPosition(m_Sound, Pos);
}

////////////////////////////////////////////////////////////////////////////////

bool
CSound::IsPlaying() const
{
  return SA_IsPlaying(m_Sound) != 0;
}

////////////////////////////////////////////////////////////////////////////////
