#include <windows.h>
#include <stdio.h>
#include "win32_audio.hpp"
#include "win32_internal.hpp"
#include "win32_sphere_config.hpp"


static bool  (__stdcall * SA_InitDriver)(HWND window);
static void  (__stdcall * SA_CloseDriver)();

void  (__stdcall * SA_SetPrivate)(void* data);
void* (__stdcall * SA_GetPrivate)();
void  (__stdcall * SA_SetFileCallbacks)(
  SA_FileOpenCallback  open,
  SA_FileCloseCallback close,
  SA_FileReadCallback  read,
  SA_FileSeekCallback  seek,
  SA_FileTellCallback  tell,
  SA_FileSizeCallback  size);

SA_SOUND (__stdcall * SA_LoadSound)(const char* filename);
void     (__stdcall * SA_DestroySound)(SA_SOUND sound);
void     (__stdcall * SA_PlaySound)(SA_SOUND sound, bool repeat);
void     (__stdcall * SA_StopSound)(SA_SOUND sound);
void     (__stdcall * SA_SetVolume)(SA_SOUND sound, int volume);
int      (__stdcall * SA_GetVolume)(SA_SOUND sound);
int      (__stdcall * SA_GetLength)(SA_SOUND sound);
void     (__stdcall * SA_SetPosition)(SA_SOUND sound, int position);
int      (__stdcall * SA_GetPosition)(SA_SOUND sound);
int      (__stdcall * SA_IsPlaying)(SA_SOUND sound);


static HINSTANCE AudioDriver;


////////////////////////////////////////////////////////////////////////////////

template<typename T>
void assign(T& dest, void* src)
{
  dest = reinterpret_cast<T&>(src);
}

bool InitAudio(HWND window, SPHERECONFIG* config)
{
  char audiodriver[MAX_PATH];
  sprintf(audiodriver, "system/audio/%s", config->audiodriver.c_str());

  // Loads driver
  AudioDriver = LoadLibrary(audiodriver);
  if (AudioDriver == NULL)
    return false;

  // gets addresses of all of the graphics functions
#define get(x) {                              \
  assign(x, GetProcAddress(AudioDriver, #x)); \
  if (x == NULL) {                            \
    FreeLibrary(AudioDriver);                 \
    return false;                             \
  }                                           \
}
  
  get(SA_InitDriver)
  get(SA_CloseDriver)

  get(SA_SetPrivate)
  get(SA_GetPrivate)
  get(SA_SetFileCallbacks)
  get(SA_LoadSound)
  get(SA_DestroySound)
  get(SA_PlaySound)
  get(SA_StopSound)
  get(SA_SetVolume)
  get(SA_GetVolume)
  get(SA_GetLength)
  get(SA_SetPosition)
  get(SA_GetPosition)
  get(SA_IsPlaying)

#undef get

  // Initializes driver
  if (SA_InitDriver(window) == false)
  {
    FreeLibrary(AudioDriver);
    return false;
  }

  return true;
}

////////////////////////////////////////////////////////////////////////////////

void CloseAudio()
{
  if (AudioDriver)
  {
    SA_CloseDriver();
    FreeLibrary(AudioDriver);
  }
}

////////////////////////////////////////////////////////////////////////////////
